class Colors:
    def __new__(self, e = True, *_arg, **args) -> str:
        self.enabled = e
        
        self.fgKeys = ['foreground', 'fg', 'f']
        self.bgKeys = ['background', 'bg', 'b']
        self.decKeys = ['decorations', 'decor', 'deco', 'dec', 'd']
        self.tKeys = ['text', 't']
        self.pKeys = ['preset', 'p']
        
        self.default = ['9', 'def', 'res', 'default', 'auto']
        self.black = ['0', 'bl', 'x', 'black']
        self.red = ['1', 'r', 'red']
        self.green = ['2', 'g', 'green']
        self.yellow = ['3', 'y', 'yellow']
        self.blue = ['4', 'b', 'blue']
        self.magenta = ['5', 'm', 'p', 'magenta', 'pink', 'purple']
        self.cyan = ['6', 'c', 'lb', 'cyan', 'lightblue']
        self.white = ['7', 'w', 'white']
        
        self.bold = ['1', 'bold', 'black', 'b']
        self.italic = ['3', 'italic', 'i']
        self.underline = ['4', 'underline', 'link', 'url', 'uri', 'u', 'l', '_']
        self.strike = ['9', 'strikethrough', 'strike', 'cross', 'dash', 'del', 's', 'd']
        
        self.escapeCode = '\033['      
        self.res = '\x1b[0m'
        self.clearEscapeCode = self.escapeCode
        
        self.foreground = False
        self.background = False
        self.decorations = False
        self.text = False
        self.preset = False
        
        self.fgVal = None
        self.bgVal = None
        self.decVal = None
        self.tVal = None
        self.pVal = None
        
        if not self.enabled:
            return ''
        
        for arg, val in args.items():
            if arg in self.fgKeys:
                self.foreground = True
                self.fgVal = val
                
            if arg in self.bgKeys:
                self.background = True
                self.bgVal = val
                
            if arg in self.decKeys:
                self.decorations = True
                self.decVal = val
                
            if arg in self.tKeys:
                self.text = True
                self.tVal = val
                
            if arg in self.pKeys:
                self.preset = True
                self.pVal = val
        
        if self.foreground:
            if self.fgVal in self.default:
                self.__makeFg__(self, self.default[0])
                
            if self.fgVal in self.black:
                self.__makeFg__(self, self.black[0])    
                
            if self.fgVal in self.red:
                self.__makeFg__(self, self.red[0])
                
            if self.fgVal in self.green:
                self.__makeFg__(self, self.green[0])
                
            if self.fgVal in self.yellow:
                self.__makeFg__(self, self.yellow[0])
                    
            if self.fgVal in self.blue:
                self.__makeFg__(self, self.blue[0])
                
            if self.fgVal in self.magenta:
                self.__makeFg__(self, self.magenta[0])
                
            if self.fgVal in self.cyan:
                self.__makeFg__(self, self.cyan[0])
                
            if self.fgVal in self.white:
                self.__makeFg__(self, self.white[0])                     
                
        if self.background:
            if self.bgVal in self.default:
                self.__makeBg__(self, self.default[0])
                
            if self.bgVal in self.black:
                self.__makeBg__(self, self.black[0])    
                
            if self.bgVal in self.red:
                self.__makeBg__(self, self.red[0])
                
            if self.bgVal in self.green:
                self.__makeBg__(self, self.green[0])
                
            if self.bgVal in self.yellow:
                self.__makeBg__(self, self.yellow[0])
                    
            if self.bgVal in self.blue:
                self.__makeBg__(self, self.blue[0])
                
            if self.bgVal in self.magenta:
                self.__makeBg__(self, self.magenta[0])
                
            if self.bgVal in self.cyan:
                self.__makeBg__(self, self.cyan[0])
                
            if self.bgVal in self.white:
                self.__makeBg__(self, self.white[0])
                
        if self.decorations:
            decorations = self.decVal.split(' ')
            for decorator in decorations:
                if decorator in self.bold:
                    self.__makeDec__(self, self.bold[0])
                    
                if decorator in self.italic:
                    self.__makeDec__(self, self.italic[0])
                
                if decorator in self.underline:
                    self.__makeDec__(self, self.underline[0])
                    
                if decorator in self.strike:
                    self.__makeDec__(self, self.strike[0])
                    
        if self.preset:
            if self.pVal == 'link':
                self.__clearEscapeCode__(self)
                self.__makeFg__(self, self.blue[0])
                self.__makeDec__(self, self.italic[0])
                self.__makeDec__(self, self.underline[0])
                
            if self.pVal == 'error':
                self.__clearEscapeCode__(self)
                self.__makeFg__(self, self.red[0])
                self.__makeDec__(self, self.italic[0])
                
        if self.text:
            return self.__finalEscapeCode__(self) + self.tVal + self.res
        
            
        return self.__finalEscapeCode__(self)


    def __makeFg__(self, color:str) -> None:
        self.escapeCode += f'3{color}'

        
    def __makeBg__(self, color:str) -> None:
        self.escapeCode += f';4{color}'

        
    def __makeDec__(self, dec:str) -> None:
        self.escapeCode += f';{dec}'
        
    
    def __clearEscapeCode__(self) -> None:
        self.escapeCode = self.clearEscapeCode
        
        
    def __finalEscapeCode__(self) -> str:
        return self.escapeCode + 'm'


import datetime
import re

from ColadaUtility.PowerColor import Colors as c
res = 'res'

class Logger():
    def __init__(self) -> None:
        self.writeDateToTerminal = True
        self.writeDataToFile = True
        self.coloredTerminal = False
        self.outputFile = ''
        self.outputFiles = []
        self.tune()
    
    
    def tune(self) -> None:
        if self.outputFile != '':
            self.outputFiles.append(self.outputFile)
        self.encoding = 'utf-8'
        self.dateFormat = f'%Y-%m-%d ||{c(e=self.coloredTerminal, f="y")} %H:%M:%S{c()} || '
    
    
    def d(self, inf: str) -> None:
        message =  f'{self.__getDate__()}{c(e=self.coloredTerminal, f="lb", d="b", text="DEBUG")} : {inf}' 
        terminalMessage, fileMessage = self.__generateMessage__(message)
        print(terminalMessage)
        self.__logToFiles__(fileMessage)
        
        
    def i(self, inf: str) -> None:
        message = f'{self.__getDate__()}{c(e=self.coloredTerminal, f="y", d="b", text=" INFO")} : {inf}'
        terminalMessage, fileMessage = self.__generateMessage__(message)
        print(terminalMessage)
        self.__logToFiles__(fileMessage)
        
    
    def w(self, inf: str) -> None:
        message = f'{self.__getDate__()}{c(e=self.coloredTerminal, f="p", d="b", text=" WARN")} : {inf}'
        terminalMessage, fileMessage = self.__generateMessage__(message)
        print(terminalMessage)
        self.__logToFiles__(fileMessage)


    def e(self, inf: str, displaySeeFile: bool = False) -> None:
        message = f'{self.__getDate__()}{c(e=self.coloredTerminal, f="r", b="x", d="b", text="ERROR")} : ' +\
        f'{c(e=self.coloredTerminal, p="error")}{inf}{c()}'
        terminalMessage, fileMessage = self.__generateMessage__(message)
        seeFileCaution = f'{c(p="error")}, see {c(p="link")}{self.outputFiles[0]}{c()} {c(p="error")}for details{c()}'
        
        if self.writeDataToFile and len(self.outputFiles) > 0 and displaySeeFile:
            terminalMessage += seeFileCaution
            
        print(terminalMessage)
        self.__logToFiles__(fileMessage)
        
    def f(self, inf: str) -> None:
        message = f'{self.__getDate__()}{c(e=self.coloredTerminal, f="r", b="x", d="b", text="HIDEN")} : ' +\
        f'{c(e=self.coloredTerminal, f="r", d="i")}{inf}{c()}'
        terminalMessage, fileMessage = self.__generateMessage__(message)
        self.__logToFiles__(fileMessage)
                    
    
    def __logToFiles__(self, message: str) -> None:
        for _file in self.outputFiles:
            try:
                with open(_file, mode='r') as f:
                    f.close()
            
            except(FileNotFoundError):
                self.__createLogFile__(_file)
                
            finally:
                with open(_file, mode='a', encoding=self.encoding) as f:
                    f.write(message + '\n')
                    f.close()
                    
    
    def __createLogFile__(self, filepath: str) -> None: 
        try:
            with open(filepath, mode='w') as f:
                f.close()
                self.i(f'Created file {filepath}')
                
        except:
            self.i(f'Unable to create file: {filepath}')
    
                    
    def __generateMessage__(self, message:str):
        terminal = message
        filem = re.sub(r'([^m]*)m', '', message)
        
        return terminal, filem                
                
    def __getDate__(self) -> str:
        date = datetime.datetime.now().strftime(self.dateFormat)       
        return str(date)
    



import unittest
from ColadaUtility.PowerColor import Colors as c

class ColorTest(unittest.TestCase):
    def test_foreground(self):
        self.assertEqual(c(f='r'), '\033[31m')
        
    def test_background(self):
        self.assertEqual(c(b='x'), '\033[;40m')
        
    def test_decorations(self):
        self.assertEqual(c(d='b i u s'), '\033[;1;3;4;9m')
        
    def test_text(self):
        self.assertEqual(c(f='y', b='b', d='u', t='test'), '\033[33;44;4mtest\033[0m')
        

if __name__ == '__main__':
    unittest.main()


import winreg

class Proxy():
    def __init__(self) -> None:
        self.__regpath__ = r'Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings'
    
    def set(self, addr: str) -> bool:
        try:
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                                 self.__regpath__,
                                 0, winreg.KEY_SET_VALUE)
            winreg.SetValueEx(key, 'ProxyServer', 0, 1, addr)
            winreg.CloseKey(key)
            return True
        
        except:
            return False
        
    
    def enable(self) -> bool:
        try:
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                                 self.__regpath__,
                                 0, winreg.KEY_SET_VALUE)
            winreg.SetValueEx(key, 'ProxyEnable', 0, 4, 1)
            winreg.CloseKey(key)
            return True
        
        except:
            return False
        
    
    def disable(self) -> bool:
        try:
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                                 self.__regpath__,
                                 0, winreg.KEY_SET_VALUE)
            winreg.SetValueEx(key, 'ProxyEnable', 0, 4, 0)
            winreg.CloseKey(key)
            return True
        
        except:
            return False
        
    
    def getCurrentProxy(self) -> str:
        try:
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                                 self.__regpath__,
                                 0, winreg.KEY_READ)
            proxy = winreg.QueryValueEx(key, 'ProxyServer')
            winreg.CloseKey(key)
            return proxy[0]
        except:
            return 'Failed to get current proxy'
        

    def getProxyStatus(self) -> bool:
        try:
            key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                                 self.__regpath__,
                                 0, winreg.KEY_READ)
            status = winreg.QueryValueEx(key, 'ProxyEnable')
            winreg.CloseKey(key)
            return bool(status[0])
        
        except:
            raise(RuntimeError)